Program SimulerDifferentialProces;

Type
   ProcesDataType = Record
      K:         Real;        { Proceskonstant     }
      Tau:       Real;        { Procestidskonstant }
      Cn:        Real;        { Procesoutput       }
   end;

{-----------------------------------------------------------}
{ Initiering af Proces ved opstart                          }
{-----------------------------------------------------------}
Procedure InitProces ( Var P: ProcesDataType; K, Tau: Real );
begin
   P.K   := K;          { Proceskonstant }
   P.Tau := Tau;        { Procestidskonstant }
   P.Cn  := 0;          { Procesoutput       }
end;

{----------------------------------------------------------}
{ Beregning af et proces-trin (ny output ved sampling)     }
{----------------------------------------------------------}
Procedure Proces ( Var P: ProcesDataType;
                   Mn, T: Real; Var Cn: Real);
Var
   ExpTTau: Real;
   Mn_1:    Real;
   Cn_1:    Real;

begin

   Mn_1  := Mn;      { BEMRK der gr en sampling fr  }
                     { processen er slut og ny output  }
   Cn_1  := P.Cn;    { kan beregnes                    }

   ExpTTau := Exp(-T/P.Tau);            { Beregn }
   Cn := (ExpTTau * Cn_1) +             { ny     }
         (P.K * (1-ExpTTau) * Mn_1);    { output }

   P.Cn := Cn;       { Gem proceoutput til nste gang }

end;

{-----------------------------------------------------------}
{  Simulering af proces med stepinput                       }
{-----------------------------------------------------------}
Var

   PData:       ProcesDataType;
   SamplTid:    Real;
   PKonst:      Real;          { Proceskonstant }
   PTidsKonst:  Real;          { Proces tidskonstant }
   PInput:      Real;          { = Controleroutput }
   POutput:     Real;
   Sampling:    Integer;
   AntalSampl:  Integer;
   StartPuls:   Integer;
   PulsLength:  Integer;
   DataLog:     Text;

Begin

   Assign(DataLog,'DifProc.PRN');
   Rewrite(DataLog);

   SamplTid   := 1.0;     { Tid mellem samplinger }
   POutput    := 0;
   PKonst     := 1.8;
   PTidsKonst := 2.5;
   InitProces (PData, PKonst, PTidsKonst);

   StartPuls  := 3;
   PulsLength := 20;
   AntalSampl := 40;

   For Sampling := 0 to AntalSampl do begin

      if (Sampling >= StartPuls) AND
         (Sampling < StartPuls+PulsLength) then
          Pinput := 1
      else
          Pinput := 0;

      WriteLN(DataLog,Sampling:4, PInput:20, POutput:20);

      Proces (PData ,PInput, SamplTid, POutput);


   End;

   Close(DataLog);

End.
