Program SimulerDobbeltIntegralProces;

Type
   ProcesDataType = Record
      K:                  Real;    { Proceskonstant         }
      Mn: Array[-2..0] of Real;    { Procesinput            }
      Cn: Array[-2..0] of Real;    { Procesoutput           }
   end;

{-----------------------------------------------------------}
{ Initiering af Proces ved opstart                          }
{-----------------------------------------------------------}
Procedure InitProces ( Var P: ProcesDataType; K: Real );
Var
   i: Integer;
begin
   P.K   := K;          { Proceskonstant }
   For i := -2 to 0 do begin
      P.Mn[i] := 0;     { Procesinput  }
      P.Cn[i] := 0;     { Procesoutput }
   end;
end;

{----------------------------------------------------------}
{ Beregning af et proces-trin (ny output ved sampling)     }
{----------------------------------------------------------}
Procedure Proces ( Var P: ProcesDataType;
                   Mn, T: Real; Var Cn: Real);
Var
   i: Integer;
begin
   P.Mn[0] := Mn;    { Gem procesinput til senere brug }
   Cn  := 2*P.Cn[-1] - P.Cn[-2] +
          0.5*(P.K*T*T*P.Mn[-1]) + 0.5*(P.K*T*T*P.Mn[-2]);
   P.Cn[0] := Cn;    { Gem procesout til senere brug }
   For i := -2 to -1 do begin
      P.Mn[i] := P.Mn[i+1];
      P.Cn[i] := P.Cn[i+1];
   end;
end;

{-----------------------------------------------------------}
{  Simulering af proces med stepinput                       }
{-----------------------------------------------------------}
Var
   PData:       ProcesDataType;
   Sampling:    Integer;
   AntalSampl:  Integer;
   SamplTid:    Real;
   PKonst:      Real;          { Proceskonstant }
   PInput:      Real;          { = Controleroutput }
   POutput:     Real;
   DataLog:     Text;

Begin
   Assign(DataLog,'DataLog.PRN');
   Rewrite(DataLog);

   SamplTid   := 1;     { Tid mellem samplinger }
   PKonst     := 0.1;
   InitProces (PData, PKonst);

   AntalSampl := 40;
   PInput     := 1;     { Stepinput i sampling 0 }

   For Sampling := 0 to AntalSampl do begin
      Proces (PData ,PInput, SamplTid, POutput);
      WriteLN(DataLog,Sampling:4, PInput:20, POutput:20);
   End;

   Close(DataLog);
End.


