Program SimulerIntegraleProces;

Type

   ProcesDataType = Record
      K:         Real;        { Proceskonstant     }
      Cn_1:      Real;        { Procesoutput sidst }
   end;

   PControlerDataType = Record
      Kp:          Real;        { Controlerkonstant  }
   end;
   IControlerDataType = Record
      Ki:          Real;        { Controlerkonstant  }
      Mn_1:        Real;        { Controleroutput sidst }
   end;

{-----------------------------------------------------------}
{ Initiering af Proces ved opstart                          }
{-----------------------------------------------------------}
Procedure InitProces ( Var P: ProcesDataType; K: Real );
begin
   P.K   := K;          { Proceskonstant }
   P.Cn_1 := 0;         { Procesoutput sidste gang  }
end;

{----------------------------------------------------------}
{ Beregning af et proces-trin (ny output ved sampling)     }
{----------------------------------------------------------}
Procedure Proces ( Var P: ProcesDataType;
                   Mn_1, T: Real; Var Cn: Real);
begin
   Cn  := P.Cn_1 + (P.K * T * Mn_1);
   P.Cn_1 := Cn;   { Gem proceoutput til nste gang }
end;

{-----------------------------------------------------------}
{ Initiering af Controler ved opstart                       }
{-----------------------------------------------------------}
Procedure InitPControler (Var C: PControlerDataType;
                          Kp: Real);
begin
   C.Kp  := Kp;        { Controlerkonstant            }
end;

{-----------------------------------------------------------}
{ Beregning af et controler-trin (ny output ved sampling)   }
{-----------------------------------------------------------}
Procedure PControler ( Var C: PControlerDataType;
                       En: Real; Var Mn: Real);
begin
   Mn := C.Kp * En;
end;

{-----------------------------------------------------------}
{ Initiering af Controler ved opstart                       }
{-----------------------------------------------------------}
Procedure InitIControler (Var C: IControlerDataType;
                          Ki: Real);
begin
   C.Ki   := Ki;         { Controlerkonstant            }
   C.Mn_1 := 0;          { Controleroutput sidste gang  }
end;

{-----------------------------------------------------------}
{ Beregning af et controler-trin (ny output ved sampling)   }
{-----------------------------------------------------------}
Procedure IControler ( Var C: IControlerDataType;
                       En, T: Real; Var Mn: Real);
begin
   Mn := C.Mn_1 + (C.Ki * T * En);
   C.Mn_1 := Mn;   { Gem controleroutput til nste gang }
end;

{-----------------------------------------------------------}
{  Simulering af proces med stepinput                       }
{-----------------------------------------------------------}
Var
   PData:       ProcesDataType;
   PCData:      PControlerDataType;
   ICData:      IControlerDataType;
   SamplTid:    Real;
   PKonst:      Real;      { Proceskonstant }
   PCKonst:     Real;      { proportionalcontroler konstant }
   ICKonst:     Real;      { integralcontroler konstant }
   PInput:      Real;      { = Controleroutput }
   POutput:     Real;
   Reference:   Real;      { nsket procesoutput }
   Fejl:        Real;
   Sampling:    Integer;
   AntalSampl:  Integer;
   StartPuls:   Integer;
   PulsLength:  Integer;
   DataLog:     Text;
   Tast:        Char;

Begin

   Assign(DataLog,'DataLog.PRN');
   Assign(DataLog,'CON');
   Rewrite(DataLog);

   PKonst := 1;
   InitProces (PData, PKonst);

   Write('Indtast samplingstid:                 '); ReadLN(Sampltid);
   Write('Indtast proportionalcontrolerkonstant:'); ReadLN(PCKonst);
   Write('Indtast integralcontrolerkonstant:    '); ReadLN(ICKonst);
   InitPControler (PCData, PCKonst);
   InitIControler (ICData, ICKonst);

   Reference := 1;
   PInput    := 0;

   Sampling  := 0;

{   AntalSampl := 40;
   For Sampling := 0 to AntalSampl do begin
}
   Repeat

      Proces (PData ,PInput, Sampltid, POutput);

      Fejl := Reference - POutput;
      PControler (PCData, Fejl, PInput);

      WriteLN(DataLog,'S:',Sampling:4,' R:',Reference:7:3,
                      ' E:',Fejl:10:3,
                      ' M:',PInput:10:3,' C:',POutput:10:3);
{
      WriteLN(DataLog,Sampling:4,Reference:20,
                      PInput:20,POutput:20,Fejl:20);
}
{
      IControler (ICData, Fejl, Sampltid, PInput);
}
      Sampling := Sampling + 1;

      {Tast := ReadKey;} ReadLN(tast);

   Until Tast = 'S';

   Close(DataLog);

End.

Type
.................
.................
Var
   PControlerData: PControlerDataType;
   SamplingsTid:      Real;
   Reference:         Real;          { nsket procesoutput }
   PControlerKonst:   Real;
   Fejl:              Real;
   ProcesInput:       Real;          { = Controleroutput }
   .................
   .................
   .................
   .................
   SamplingsTid    := 1;
   PControlerKonst := 0.5;
   InitPControler (PControlerData, PControlerKonst);
   .................
   .................
   .................
   .................
   Fejl := Reference - ProcesOutput;
   PControler (PControlerData, Fejl, ProcesInput);
