Program SimulerStyretSystem2B;

Type
   PControlerDataType = Record
      Kp:          Real;        { Controlerkonstant  }
   end;
   IControlerDataType = Record
      Ki:          Real;        { Controlerkonstant  }
      Mn_1:        Real;        { Controleroutput sidst }
   end;
   PIControlerDataType = Record
      Kp:   Real;   { Controlerkonstant proportionaldel }
      Ki:   Real;   { Controlerkonstant integraledel }
      En_1: Real;   { Fejl sidst }
      Mn_1: Real;   { Controleroutput sidst }
   end;
   PIDControlerDataType = Record
      Kp:   Real;   { Controlerkonstant proportionaldel }
      Ki:   Real;   { Controlerkonstant integraledel }
      Kd:   Real;   { Controlerkonstant differeintialdel }
      En_2: Real;   { Fejl 2 perioder tilbage }
      En_1: Real;   { Fejl sidst }
      Mn_1: Real;   { Controleroutput sidst }
   end;

   ProcesDataType = Record
      K:         Real;        { Proceskonstant     }
      Cn:        Real;        { Procesoutput       }
   end;
{-----------------------------------------------------------}
{ Initiering af proportionalcontroler ved opstart           }
{-----------------------------------------------------------}
Procedure InitPControler (Var C: PControlerDataType;
                          Kp: Real);
begin
   C.Kp  := Kp;        { Controlerkonstant            }
end;

{-----------------------------------------------------------}
{ Beregning af et proportional-controler-trin (ny styring)  }
{-----------------------------------------------------------}
Procedure PControler ( Var C: PControlerDataType;
                       En: Real; Var Mn: Real);
begin
   Mn := C.Kp * En;
end;

{-----------------------------------------------------------}
{ Initiering af integralcontroler ved opstart               }
{-----------------------------------------------------------}
Procedure InitIControler (Var C: IControlerDataType;
                          Ki: Real);
begin
   C.Ki   := Ki;         { Controlerkonstant            }
   C.Mn_1 := 0;          { Controleroutput sidste gang  }
end;

{-----------------------------------------------------------}
{ Beregning af et integral-controler-trin (ny styring)      }
{-----------------------------------------------------------}
Procedure IControler ( Var C: IControlerDataType;
                       En, T: Real; Var Mn: Real);
begin
   Mn := C.Mn_1 + (C.Ki * T * En);
   C.Mn_1 := Mn;   { Gem controleroutput til nste gang }
end;

{-----------------------------------------------------------}
{ Initiering af proportional-integra-lcontroler             }
{-----------------------------------------------------------}
Procedure InitPIDControler (Var C: PIDControlerDataType;
                            Kp, Ki, Kd: Real);
begin
   C.Kp  := Kp;      { Controlerkonstant proportionaldel }
   C.Ki  := Ki;      { Controlerkonstant integraledel }
   C.Kd  := Kd;      { Controlerkonstant differentialdel }
   C.En_1 := 0;      { Fejl sidste gang  }
   C.Mn_1 := 0;      { Controleroutput sidste gang  }
end;
{-----------------------------------------------------------}
{ Beregning af et integral-differential- controler-trin     }
{    (ny output ved sampling)                               }
{-----------------------------------------------------------}
Procedure PIDControler ( Var C: PIDControlerDataType;
                         En, T: Real; Var Mn: Real);
begin
   Mn := C.Mn_1 + (C.Kp + (C.Ki*T) + (C.Kd/T))*En -
         (C.Kp + 2*(C.Kd/T)*C.En_1 + (C.Kd/T)*C.En_2);
   C.Mn_1 := Mn;      { Gem controleroutput til nste gang }
   C.En_2 := C.En_1;  { Gem fejl til nste gang }
   C.En_1 := En;      { Gem fejl til nste gang }
end;

{-----------------------------------------------------------}
{ Initiering af Proces ved opstart                          }
{-----------------------------------------------------------}
Procedure InitProces ( Var P: ProcesDataType; K: Real );
begin
   P.K  := K;         { Proceskonstant }
   P.Cn := 0;         { Procesoutput   }
end;

{----------------------------------------------------------}
{ Beregning af et proces-trin (ny output ved sampling)     }
{----------------------------------------------------------}
Procedure Proces ( Var P: ProcesDataType;
                   Mn, T: Real; Var Cn: Real);
Var
   Mn_1:  Real;
   Cn_1:  Real;

begin
   Mn_1 := Mn;     { BEMRK der gr en sampling fr  }
                   { processen er slut og ny output  }
   Cn_1 := P.Cn;   { kan beregnes                    }

   Cn   := Cn_1 + (P.K * T * Mn_1);  { Beregn output }

   P.Cn := Cn;     { Gem proceoutput til nste gang }
end;

{-----------------------------------------------------------}
{  Simulering af proces med stepinput                       }
{-----------------------------------------------------------}
Var
   PData:       ProcesDataType;
   PCData:      PControlerDataType;
   ICData:      IControlerDataType;
   SamplTid:    Real;
   PKonst:      Real;      { Proceskonstant }
   PCKonst:     Real;      { proportionalcontroler konstant }
   ICKonst:     Real;      { integralcontroler konstant }
   PInput:      Real;      { = Controleroutput }
   POutput:     Real;
   Reference:   Real;      { nsket procesoutput }
   Fejl:        Real;
   Sampling:    Integer;
   AntalSampl:  Integer;
   StartPuls:   Integer;
   PulsLength:  Integer;
   DataLog:     Text;
   Tast:        String[4];
   Test:        Integer;
   
Begin

   Assign(DataLog,'DataLog.PRN');
   Assign(DataLog,'CON');
   Rewrite(DataLog);

   PKonst := 1;
   InitProces (PData, PKonst);

   Write('Indtast samplingstid:                 '); ReadLN(Sampltid);
   Write('Indtast proportionalcontrolerkonstant:'); ReadLN(PCKonst);
   Write('Indtast integralcontrolerkonstant:    '); ReadLN(ICKonst);
   InitPControler (PCData, PCKonst);
   InitIControler (ICData, ICKonst);

   Reference := 1;
   POutput   := 0;

   Sampling  := 0;

   AntalSampl := 40;

   ReadLN(tast);
   While (Tast <> 'S') AND (Sampling < AntalSampl) do begin
      Val(Tast,Reference,Test);
      Fejl := Reference - POutput;
{      PControler (PCData, Fejl, PInput);
}
      IControler (ICData, Fejl, Sampltid, PInput);
      WriteLN(DataLog,'S=',Sampling:4,
                      ', R=',Reference:7:3,
                      ', C=',POutput:10:3,
                      ', E=',Fejl:10:3,
                      ', M=',PInput:10:3);
      Sampling := Sampling + 1;
      Proces (PData ,PInput, Sampltid, POutput);

      ReadLN(tast);

   end;

   Close(DataLog);

End.
